



echo on
IF %ERRORLEVEL% NEQ 0 exit /B 1
cd test
IF %ERRORLEVEL% NEQ 0 exit /B 1
mkdir build_default
IF %ERRORLEVEL% NEQ 0 exit /B 1
cd build_default
IF %ERRORLEVEL% NEQ 0 exit /B 1
cmake .. -G"%CMAKE_GENERATOR%" -DCMAKE_BUILD_TYPE=Release
IF %ERRORLEVEL% NEQ 0 exit /B 1
cmake --build . --config Release
IF %ERRORLEVEL% NEQ 0 exit /B 1
Release\\program.exe
IF %ERRORLEVEL% NEQ 0 exit /B 1
cd ..
IF %ERRORLEVEL% NEQ 0 exit /B 1
mkdir build_ninja
IF %ERRORLEVEL% NEQ 0 exit /B 1
cd build_ninja
IF %ERRORLEVEL% NEQ 0 exit /B 1
cmake .. -G"Ninja" -DCMAKE_BUILD_TYPE=Release
IF %ERRORLEVEL% NEQ 0 exit /B 1
cmake --build . --config Release
IF %ERRORLEVEL% NEQ 0 exit /B 1
program.exe
IF %ERRORLEVEL% NEQ 0 exit /B 1
exit /B 0
